# -*- coding: utf-8 -*-

# Define your item pipelines here
#
# Don't forget to add your pipeline to the ITEM_PIPELINES setting
# See: http://doc.scrapy.org/en/latest/topics/item-pipeline.html
from datetime import datetime
import json
from scrapy.utils.project import get_project_settings
import redis
from newspaper import Article

from .classificator.news_classify_tag import Classifier
from .helpers.db import Database
from .helpers.utilits import MLDataMaker, News

db = get_project_settings().get('DB')


class ExaPipeline(object):
    def __init__(self):
        '''
        This pipeline make
        conect to DB,
        create classifier for making tags
        make Redis buffer for collect all news from DB
        '''
        self.teach = False
        self.db = Database(**db)
        self.classifier = None
        self.buffer = redis.StrictRedis()
        self.ml_data = MLDataMaker(self.db)
        for i in self.db.select('select url from wp_esi_news_accept'):
            self.buffer.set(i[0], True)
        self.tag_radar = {i[0]: i[1] for i in self.db.select('select tag_id, radar_id from wp_esi_radar_tag')}
        super(ExaPipeline, self).__init__()

    def open_spider(self, spider):
        spider.pipeline = self
        self.teach = spider.teach
        self.classifier = self._create_classifier()

    def process_item(self, item, spider):
        item['title'] = ''.join(item['title']).replace('\n', ' ')
        if item['description']:
            item['description'] = ''.join(item['description']).replace('\n', ' ')
        if item['tags']:
            item['tags'] = ','.join(item['tags']).replace('\n', '').replace('\t', '')

        if self.check_url(item['url']):
            print("DUPLICATE")
        else:
            print("UNIQUE")
            article = self.get_article(item['url'])
            text = article.text
            tags = self.get_tags(text)
            radar = self.get_radar(tags)
            tags = json.dumps(tags) if tags else None
            radar = json.dumps(radar) if radar else None
            news = self.insert_news(item, tags, radar, text)
            self.update_ml_data(news, item['tags'])
        return item

    def insert_news(self, item, tags, radar, text):
        data = (item['title'], item['description'], item['url'], item['media_id'], item['type_id'],
                item['region_id'], item['post_id'], item['date'], datetime.now(), item['company_id'], 0,
                item['tags'], tags, text, radar)
        query = """INSERT INTO wp_esi_news_accept (title, description, URL, media_id, type_id, region_id, post_id,
                publish_date, record_date, company_id, is_accepted, temp_tags, tags_id, text, radar_id) 
                VALUES(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s);\n"""

        news = self.db.insert(query, data)
        self.add_url_to_block(item['url'])
        return news

    def check_url(self, url):
        '''
        Function check url in redis buffer
        :param url: url from item
        :return: True if url already exist in buffer
        '''
        if bool(self.buffer.get(url)):
            print("IN REDIS")
            return True
        else:
            return False

    def add_url_to_block(self, url):
        self.buffer.set(url, True)

    def get_article(self, url):
        '''
        Function take text from url for processing
        :param url:
        :return: text from url
        '''
        article = Article(url)
        article.download()
        article.parse()
        return article

    def get_tags(self, text):
        '''
        Function for make tags use ML algorithm
        :param text: Text for searching tags
        :return: json object with tags that are relevant for text
        '''
        self.classifier.classify(text)
        tags = list()
        for i in self.classifier.tag_accordance:
            tags.append(i[0])
        if len(tags) > 0:
            return tags
        return None

    def get_radar(self, tags):
        if tags:
            radar = [self.tag_radar[i] for i in tags if i in self.tag_radar]
            if len(radar) > 0:
                return radar
        return None

    def update_ml_data(self, news, tags):
        news = News(news, tags)
        data = self.ml_data.make_ml_data(news)
        if data:
            for i in data:
                self.db.insert('INSERT INTO wp_esi_ml_data (news_id, tag_id) VALUES(%s, %s);',
                               (i.news_id, i.tags_id))

    def _create_classifier(self):
        if self.teach:
            classifier = Classifier(host=db['host'], user=db['user'], password=db['pwd'], db=db['database'], port=3306)
            classifier.teach_model()
            return classifier
        else:
            return None
